/*
 * $Id: sbrg_live.c,v 1.2 2002/03/15 17:07:36 kenta Exp $
 *
 * Copyright 2002 Kenta Cho. All rights reserved.
 */

/**
 * Speed Barricade live camera bootstrap.
 *
 * @version $Revision: 1.2 $
 */
#include "SDL.h"
#include "pieceif.h"
#include "sbrg_live.h"
#include "livemanager.h"
#include "screen.h"

#define INTERVAL 16

// Handle data inputs from USB.

static int usbReady = 0;
static int noInputCnt = 0;
static int liveStart = 0;

// An input from usb should be sent at intervals of <(FIELD_WIDTH / MOTOR_SPEED).
#define MAX_INPUT_INTERVAL 256 

static int initUSB() {
  USBCOMS ucs;
  USBCOMS *pucs = &ucs;
  if ( ismInit() ) return 0;
  if ( ismUCOpen( pucs ) || strcmp( "Sbrg", pucs->signature ) || !pucs->pistat ) return 0;
  noInputCnt = MAX_INPUT_INTERVAL;
  return 1;
}

static void closeUSB() {
  ismUCClose();
  ismExit();
}

void usbDisconnected() {
  resetGame();
  usbReady = liveStart = 0;
}

int isLiveStart() {
  return liveStart;
}

#define USB_RECEIVE_BUF_SIZE 16

#define SET_COMMAND 0
#define UPDATE_COMMAND 1
#define DESTROY_COMMAND 2
#define DEAD_COMMAND 3
#define CRASH_COMMAND 4
#define RESET_COMMAND 5

static void readUsbData() {
  USBCOMS ucs;
  USBCOMS *pucs = &ucs;
  char msgBuf[USB_RECEIVE_BUF_SIZE*2];
  ismUCGetStat( pucs );
  if ( pucs->txstat & USBCOM_STAT_TXWAIT ) {
    int len = pucs->txlen, idx = 0;
    int cmd;
    ismUCRead(msgBuf, len);
    if ( len <= 0 || len > USB_RECEIVE_BUF_SIZE ) {
      usbDisconnected();
      return;
    }
    while ( idx < len ) {
      cmd = msgBuf[idx];
      if ( cmd < SET_COMMAND || cmd > RESET_COMMAND ) {
	usbDisconnected();
	return;
      }
      switch ( cmd ) {
      case SET_COMMAND:
	setMotor(msgBuf[idx+1], msgBuf[idx+2], 
		 (msgBuf[idx+3]<<8)+msgBuf[idx+4], (msgBuf[idx+5]<<8)+msgBuf[idx+6], 
		 msgBuf[idx+7]);
	idx += 8;
	break;
      case UPDATE_COMMAND:
	updateMotor(msgBuf[idx+1], msgBuf[idx+2], 
		    (msgBuf[idx+3]<<8)+msgBuf[idx+4], (msgBuf[idx+5]<<8)+msgBuf[idx+6], 
		    msgBuf[idx+7]);
	idx += 8;
	break;
      case DESTROY_COMMAND:
	destroyWall(msgBuf[idx+1], msgBuf[idx+2]);
	idx += 3;
	break;
      case DEAD_COMMAND:
	deadMotor(msgBuf[idx+1], 
		  (msgBuf[idx+2]<<8)+msgBuf[idx+3], (msgBuf[idx+4]<<8)+msgBuf[idx+5]);
	idx += 6;
	break;
      case CRASH_COMMAND:
	crashMotor(msgBuf[idx+1], 
		   (msgBuf[idx+2]<<8)+msgBuf[idx+3], (msgBuf[idx+4]<<8)+msgBuf[idx+5]);
	idx += 6;
	break;
      case RESET_COMMAND:
	resetGame();
	idx++;
	break;
      }
    }
    noInputCnt = MAX_INPUT_INTERVAL;
    liveStart = 1;
  } else {
    noInputCnt--;
    if ( noInputCnt <= 0 ) usbDisconnected();
  }
}


static void initSbrglive() {
  usbReady = initUSB();
}

static void closeSbrglive() {
  closeUSB();
  closeSDL();
}

// Parse command args.

static int windowMode = 0;

static void usage(char *argv0) {
  fprintf(stderr, "Usage: %s [-window]\n", argv0);
}

static void parseArgs(int argc, char *argv[]) {
  int i;
  for ( i=1 ; i<argc ; i++ ) {
    if ( strcmp(argv[i], "-window") == 0 ) {
      windowMode = 1;
    } else {
      usage(argv[0]);
      exit(1);
    }
  }
}


static int cnt = 0;

int main(int argc, char **argv) {
  int done = 0;
  long prvTickCount = 0;
  int i;

  parseArgs(argc, argv);

  initSDL(windowMode);
  srand(SDL_GetTicks());
  initSbrglive();
  initLive();

  while ( !done ) {
    long nowTick = SDL_GetTicks();
    int frame = (int)(nowTick-prvTickCount) / INTERVAL;
    if ( frame <= 0 ) {
      frame = 1;
      SDL_Delay(prvTickCount+INTERVAL-nowTick);
      prvTickCount += INTERVAL;
    } else if ( frame > 5 ) {
      frame = 5;
      prvTickCount = nowTick;
    } else {
      prvTickCount += frame*INTERVAL;
    }
    for ( i=0 ; i<frame ; i++ ) {
      move();
    }
    drawGLSceneStart();
    draw();
    drawGLSceneEnd();
    if ( usbReady ) {
      readUsbData();
    } else {
      if ( (cnt&63) == 0 ) {
	usbReady = initUSB();
      }
    }

    { SDL_Event event;
      while ( SDL_PollEvent(&event) ) {
	switch ( event.type ) {
	case SDL_QUIT:
          done = 1;
	  break;
	case SDL_KEYDOWN:
	  if ( event.key.keysym.sym == SDLK_ESCAPE ) {
	    done = 1;
	  }
	  break;
	case SDL_VIDEORESIZE:
	  resized(event.resize.w, event.resize.h);
	  break;
	}
      }
    }
    cnt++;
  }
  closeSbrglive();
}
